#include "NavigationStackFactory.h"

#include "Navigation/libraries/core/constants.h"
#include "Navigation/libraries/factories/GlobalPlannerFactory.h"
#include "Navigation/libraries/factories/LocalPlannerFactory.h"
#include "Navigation/libraries/factories/SafetyControllerFactory.h"
#include "Navigation/libraries/factories/TrajectoryControllerFactory.h"
#include "Navigation/libraries/server/NavigationStack.h"

namespace armarx::nav::fac
{
    server::NavigationStack
    NavigationStackFactory::create(const aron::datanavigator::DictNavigatorPtr& params,
                                   const core::Scene& ctx)
    {
        using aron::datanavigator::DictNavigator;

        const auto getElementOrNull =
            [&params](const std::string & key) -> DictNavigator::PointerType
        {
            if (params->hasElement(key))
            {
                return DictNavigator::DynamicCast(params->getElement(key));
            }

            ARMARX_INFO << "Skipping '" << key << "'";

            return nullptr;
        };

        const auto globalPlannerCfg = getElementOrNull(core::GLOBAL_PLANNER);
        const auto localPlannerCfg = getElementOrNull(core::LOCAL_PLANNER);
        const auto trajectoryControllerCfg = getElementOrNull(core::TRAJECTORY_CONTROLLER);
        const auto safetyControllerCfg = getElementOrNull(core::SAFETY_CONTROLLER);

        return server::NavigationStack
        {
            .globalPlanner = GlobalPlannerFactory::create(globalPlannerCfg, ctx),
            .localPlanner = LocalPlannerFactory::create(localPlannerCfg, ctx),
            .trajectoryControl = TrajectoryControllerFactory::create(trajectoryControllerCfg, ctx),
            .safetyControl = SafetyControllerFactory::create(safetyControllerCfg, ctx)};
    }

    server::NavigationStack NavigationStackFactory::create(const aron::data::AronDictPtr& params,
            const core::Scene& ctx)
    {
        const auto dict = aron::datanavigator::DictNavigator::FromAronDictPtr(params);
        return create(dict, ctx);
    }

} // namespace armarx::nav::fac
